<?php

include('Rules.php');
include('Input.php');

/**
* class Turing
* Simule le fonctionnement d'une machine de Turing à partir des Rules et de l'Input
* @author Grégory Barbier & Adrian Gaudebert - adrian@gaudebert.fr
*/
class Turing
{
    private $rules = NULL;
    private $input = NULL;
    
    /**
    * SetRules
    * Assigne les règles à utiliser
    * @param $rules Objet Rules contenant les règles
    */
    public function SetRules(&$rules)
    {
        $this->rules = $rules;
    }
    
    /**
    * SetInput
    * Assigne le bandeau à utiliser
    * @param $input Objet Input contenant le bandeau
    */
    public function SetInput(&$input)
    {
        $this->input = $input;
    }
    
    /**
    * Run
    * Lance la machine de Turing avec le bandeau spécifié
    */
    public function Run()
    {
        $res = false;
        
        $currentState;
        $currentInput;
        
        if ($this->rules != NULL && $this->input != NULL)
        {
            // Initialisation
            $currentState = $this->rules->initialState;
            $currentInput = $this->input->GetCurrent();
            
            // On récupère le résultat pour l'affichage
            $res = $this->Loop($currentState, $currentInput);
        }
        
        return $res;
    }
    
    /**
    * Loop
    * Boucle principale pour l'exécution de la machine
    * Tourne jusqu'à ce que l'état final soit atteint, 
    * ou jusqu'à ce qu'une règle de transition nécessaire ne soit pas trouvée
    */
    private function Loop($currentState, $currentInput)
    {
        $continue = true;
        $result = false;
        $rule;
        
        while ($continue)
        {
            // Recherche de la règle de transition correspondant à l'état courant
            $rule = false;
            
            foreach ($this->rules->rules as $r)
            {
                if ($r->currentState == $currentState && $r->currentInput == $currentInput)
                {
                    $rule = $r;
                    break;
                }
            }
            
            if ($rule == false)
            {
                // Pas de règle de transition, on quitte
                $result = false;
                $continue = false;
            }
            else
            {
                if ($rule->nextState == $this->rules->finalState)
                {
                    // On a atteint l'état final, on quitte
                    $result = true;
                    $continue = false;
                }
                else
                {
                    $currentState = $rule->nextState;
                    $this->input->SetCurrent($rule->nextInput);
                    $this->input->Move($rule->nextDirection);
                    $currentInput = $this->input->GetCurrent();
                }
            }
        }
        
        return $result;
    }
}

?>
